/*                            HELPSYS
                             by Crispy

This library is intended to help you quickly and easily set up a tree
structure of help topics. Links to any subtopics on a page are automatically
included, as is a Back link where applicable. There is also a Close link on
every page.

Support for Cascading Style Sheets is included; just set the css var to
a text string containing CSS instructions, or specify a cache file in single
quotes to use a .css file.

If you have problems and have overridden client/Topic(), remember to call the
parent proc:
  .=..()
If you do not, the library will not function correctly.

A note on images: HelpSys will parse the HTML text you supply for <IMG>
references, and send each one to the user as required. However, if your image
resides in a subdirectory (such as demo/blue.dmi in the demonstration), you
will need to provide the subdirectory as well; so in the case of blue.dmi,
you would need to put <img src='demo/blue.dmi'> in your HTML code, rather than
<img src='blue.dmi'>.

If you wish, you may use images from the demo at no charge. Aren't I generous? :-)

*/

datum/helpsys
	var/text                   //The body of the help text in this subtopic.
	var/title                  //The title of this subtopic. Defaults to linktext.
	var/linktext               //The text that will be displayed in links to this subtopic
	var/prefix                 //Text or HTML code to be prefixed to topics.
	var/suffix                 //Text or HTML code to be suffixed to topics.
                              //For example, a copyright message or a disclaimer.
	var/css                    //A Cascading Style Sheet, either as a string or a cache file.
	var/windowsize="400x400"   //The size of the browser window.
	var/windowname="helpsys"   //The name of the browser window to use. Set this to null to
                              //make the help come up in the default, embedded window.
	var/browseopt="titlebar=0" //Any additional options to pass to browse()

	proc/display(M=usr) //Call this to display a help topic

/////////////////////////////////////////////////////////////////////////////////
//                                                                             //
/******* Library code follows; you do not need to read beyond this point *******/
//                                                                             //
/////////////////////////////////////////////////////////////////////////////////

		//Work out what to display
		var/html="<html>\n<head>\n<title>[title]</title>\n<style>\n[css]\n</style>\n</head>\n\
		<body><span class='prefix'>[prefix]</span>\n<h1>[title]</h1>\n<p><span class='maintext'>[text]</span>\n<p>"

		for (var/helptype in typesof(type)-type)
			var/datum/helpsys/subtopic=locate(helptype)
			if (!subtopic) subtopic=new helptype()
			if (subtopic.parent_type != type) continue
			html+="<a class='subtopiclink' \
			href='?src=\ref[src];help=\ref[subtopic];usr=\ref[M]'>[subtopic.linktext]</a>\n<br>"
			subtopic.tag="\ref[src]"

		if (tag) html+="<p><a class='backlink' href='?src=\ref[src];help=[tag];usr=\ref[M]'>Back</a>\n"

		html+="<br><a class='closelink' href='?src=\ref[src];close=1;usr=\ref[M]'>Close</a>\
		\n<p><span class='suffix'>[suffix]</span>\n</body>\n</html>"

		//Parse for images
		var/trimhtml=html
		while(findtext(trimhtml,"<img "))
			trimhtml=copytext(trimhtml,findtext(trimhtml,"<img ")+1)
			var/imgname=copytext(trimhtml,findtext(trimhtml," src")+4)
			//Trim spaces off image name
			while(copytext(imgname,1,2)==" ") imgname=copytext(imgname,2)
			//Trim equal sign
			if(copytext(imgname,1,2)=="=") imgname=copytext(imgname,2)
			else
				M << "Invalid image tag: [html_encode(imgname)]"
				continue //If no equal sign, this isn't a valid <img> tag, so skip it and go to the next one
			//Trim spaces
			while(copytext(imgname,1,2)==" ") imgname=copytext(imgname,2)

			//Isolate image name
			var/firstletter=copytext(imgname,1,2)
			switch(firstletter)
				if("\"","'") //double quotes (") or single quotes ('); trim to quote character
					imgname=copytext(copytext(imgname,2),1,findtext(copytext(imgname,2),firstletter))
				else //no quotes; trim to space or right angle bracket (>)
					imgname=copytext(imgname,1,findtext(imgname," "))
					imgname=copytext(imgname,1,findtext(imgname,">"))

			//Parse for cache filename, if necessary (remove any file path information)
			var/cachename=imgname
			while (findtext(cachename,"\\")) cachename=copytext(cachename,findtext(cachename,"\\")+1)
			while (findtext(cachename,"/")) cachename=copytext(cachename,findtext(cachename,"/")+1)

			//Replace image filename with cache filename in html
			if (cachename!=imgname)
				while(findtext(html,imgname))
					var/imgpos=findtext(html,imgname)
					html=copytext(html,1,imgpos)+cachename+copytext(html,imgpos+length(imgname))

			//browse_rsc() it
			if (fexists(imgname))
				var/F=file(imgname)
				M << browse_rsc(icon(F),imgname)
			else
				world.log << "HelpSys error: File '[imgname]' does not exist. Generated from [type]"

		//Display it
		M << browse(html,"window=[windowname];size=[windowsize];[browseopt]")

	New()
		//Link text and title default to each other
		if (!linktext) linktext=title
		if (!title) title=linktext

		..()

	Topic(href,href_list[])
		//Close window?
		if (href_list["close"])
			var/atom/A=locate(href_list["usr"])
			if (A) A << browse(null,"window=[windowname]")
			return

		//Help topic
		var/helpref=href_list["help"]
		if (helpref)
			var/datum/helpsys/H=locate(helpref)
			if (H) H.display()
			else CRASH("Help topic [helpref] not found")

//Copyright 2002, Crispy.